package de.intevation.artifactdatabase.rest;

import org.apache.log4j.Logger;

import org.restlet.resource.Get;  
import org.restlet.resource.ServerResource;

import org.restlet.data.Request;
import org.restlet.data.Response;
import org.restlet.data.Status;
import org.restlet.data.MediaType;

import org.restlet.representation.Representation;
import org.restlet.representation.EmptyRepresentation;

import org.restlet.ext.xml.DomRepresentation;

import de.intevation.artifactdatabase.XMLUtils;

import de.intevation.artifacts.ArtifactNamespaceContext;
import de.intevation.artifacts.ArtifactDatabaseException;
import de.intevation.artifacts.ArtifactDatabase;

import org.w3c.dom.Document;

import java.io.IOException;

/**
 * @author Sascha L. Teichmann (sascha.teichmann@intevation)
 */
public class ArtifactOutResource
extends      ServerResource
{
    public static final String PATH = "/artifact/{uuid}/{type}";

    public static final String XPATH_MIME_TYPE = "/action/out/mime-type/@value";

    public static final MediaType DEFAULT_MIME_TYPE =
        MediaType.APPLICATION_OCTET_STREAM;

    private static Logger logger = Logger.getLogger(ArtifactOutResource.class);

    @Get
    public Representation represent() {

        Request request = getRequest();

        Representation requestRepr = request.getEntity();

        Document inputDocument = null;
        try {
            DomRepresentation input = new DomRepresentation(requestRepr);
            inputDocument = input.getDocument();
        }
        catch (IOException ioe) {
            logger.error(ioe.getMessage());
            Response response = getResponse();
            response.setStatus(Status.CLIENT_ERROR_BAD_REQUEST, ioe);
            return new EmptyRepresentation();
        }

        ArtifactDatabase db = (ArtifactDatabase)getContext()
            .getAttributes().get("database");

        String identifier = (String)request.getAttributes().get("uuid");

        if (logger.isDebugEnabled()) {
            logger.debug("looking for artifact id '" + identifier + "'");
        }

        String mimeTypeString = XMLUtils.xpathString(
            inputDocument,
            XPATH_MIME_TYPE,
            ArtifactNamespaceContext.INSTANCE);

        MediaType mimeType = DEFAULT_MIME_TYPE;

        if (mimeTypeString != null && mimeTypeString.length() != 0) {
            try {
                mimeType = MediaType.valueOf(mimeTypeString);
            }
            catch (Exception e) {
                logger.error(e.getLocalizedMessage());
            }
        }

        try {
            return new OutRepresentation(
                mimeType,
                db.out(identifier, inputDocument));
        }
        catch (ArtifactDatabaseException adbe) {
            Response response = getResponse();
            response.setStatus(
                Status.CLIENT_ERROR_NOT_FOUND, adbe.getMessage());
            return new EmptyRepresentation();
        }
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8:
